/********************************************************************
  TORNADO-E3x utilities. Rev.2A
  Optional extension utilities.
  (C) MicroLAB Systems Ltd, 1996-2000

  covers:
  	E31 rev.2A
	E33 rev.1A

  Description:
  ------------
	This file contains functions for optional on-board E3x
	peripherals control (FLASH memory, etc).

	This file shall be either compiled as to .OBJ file or
	included into only one .C source file from your TE3x source
        project.

  Copyright:
  ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com
***********************************************************************/

#include "stddef.h"	// TI C Compiler standard header
#include "te3x_dsp.h"	// MicroLAB' E3x defs


/******************** general API functions ***************************

    General TE3X API functions shall be used to handle the run-time
    errors:
	TE3X_last_err
	TE3X_err_flag
	TE3X_clr_err_flag

    Each function returns error code (<0). In case the function must also
    return data (as T3X_flash_read()), then data can be recognized in case
    returned code is >=0, and error is recognized in case returned code is
    <0.
*/

static	int _te3x_last_err =0,
	    _te3x_err_flag =0;


int TE3X_last_err()
	// TE3X_LAST_ERR function returns errpr code of last executed
	// API function.
 { return(_te3x_last_err);
 }


int TE3X_err_flag()
	// TE3X_ERR_FLAG function returns status of ERR_FLAG for API
	// functions. ERR_FLAG is set by each API function in case
	// error has been detected. ERR_FLAG can be reset by
	// TE3X_CLR_ERR_FLAG API function
 { return(_te3x_err_flag);
 }

int TE3X_clr_err_flag()
 {
   _te3x_err_flag=TE3X_OFF;
   return(TE3X_OK);
 }


	// local/non-API function,
	// used for exit fm API fucntions to set error code
static int _te3x_set_err(int e)	
 {
   if(e<0)
     { _te3x_err_flag=1;
       _te3x_last_err=e;
     }
     else
       _te3x_last_err=TE3X_OK;
   return(e);
 }

//*********************************************************************


/************************** FLASH control *****************************

 IMPORTANT NOTICE !!!
 ---------------------
    As of today, FLASH control utilities for E3x have been designed for
    AMD Am29F010B/Am29F040B 5v-only FLASH memory chips only.
    These FLASH control utilities will not run perfectly with
    FLASH memory chips from other manufacturers.

    FLASH control utilities (except for the TE3X_FLASH_READ function)
    require that FLASH WRITE feature is enabled via on-board J2 jumper set,
    oherwise the TE3X_FLASH_INVALID_ERR error will be returned and no
    operation will be performed. This is due FLASH control utilies
    are auto-configured via the FLASH MANUF/DEVICE info, which is
    read from the FLASH chip using AUTO-SELECT command.

    User FLASH API functions:
    	TE3X_flash_info
	TE3X_flash_erase_chip
        TE3X_flash_erase_sector
	TE3X_flash_write
	TE3X_flash_read
	TE3X_flash_sector_prot

    At runtime, FLASH utility automatically detects the presence of the
    supported FLASH memory chips, and adjusts internal environment.

*/

#define _FLASH_AM29F010_LEN		0x20000
#define _FLASH_AM29F040_LEN		0x80000
#define _FLASH_AM29F010_SECTORS		8
#define _FLASH_AM29F040_SECTORS		8
#define _FLASH_AM29F_SECTOR_PROT_ID	1
#define _FLASH_AM29F_SECTOR_UNPROT_ID	0


static int	_te3x_flash_open_flag	= 0;
static int	_te3x_flash_manuf_id	= 0,
		_te3x_flash_dev_id	= 0,
		_te3x_flash_len		= 0,
		_te3x_flash_sectors	= 0,
		_te3x_flash_sector_len  = 0;


static int _te3x_flash_open()
	// local/non-API function, which is used to check whether
	// FLASH API has been open in order to identify FLASH chip info
	// and configure FLASH API environment
{
 unsigned *fptr,rd;
 int r;

  	// Check for FLASH API has been open already.
	// If not open, then open it.
      if (_te3x_flash_open_flag==TE3X_ON) return(TE3X_OK);	// exit if already opened

	// not open yet, now try to open FLASH utilities
      _te3x_flash_open_flag=TE3X_OFF;
      fptr = (unsigned *) TE3X_FLASH_SADDR;

       		// execute AUTO-SELECT command
       fptr[0x5555] = 0xaa;
       fptr[0x2aaa] = 0x55;
       fptr[0x5555] = 0x90;
       _te3x_flash_manuf_id = fptr[0] & TE3X_FLASH_DATA_BITMASK;

       r=TE3X_FLASH_INV_DEV_ERR;	// assume error       
       if(_te3x_flash_manuf_id != TE3X_FLASH_AMD_MANUF_ID) goto exit;

       		// now read and verify the device ID
		//-- list of supported FLASHs can be extended !!!
       _te3x_flash_dev_id = fptr[1] & TE3X_FLASH_DATA_BITMASK;
       r=TE3X_FLASH_INV_DEV_ERR;	// assume error
       if(!(         (_te3x_flash_manuf_id == TE3X_FLASH_AMD_MANUF_ID)
                && (    _te3x_flash_dev_id != TE3X_FLASH_AM29F010_DEV_ID
		     || _te3x_flash_dev_id != TE3X_FLASH_AM29F040_DEV_ID
                   )
	   )
	 )
	   goto exit;

		// now fill-in FLASH len , number of sectors, & sector len
       if(_te3x_flash_manuf_id == TE3X_FLASH_AMD_MANUF_ID)
	   {
	     switch(_te3x_flash_dev_id)
	       {
                  case TE3X_FLASH_AM29F010_DEV_ID:
			_te3x_flash_len		= _FLASH_AM29F010_LEN;
			_te3x_flash_sectors	= _FLASH_AM29F010_SECTORS;
			_te3x_flash_sector_len  = _FLASH_AM29F010_LEN/_FLASH_AM29F010_SECTORS;
			break;

                  case TE3X_FLASH_AM29F040_DEV_ID:
			_te3x_flash_len		= _FLASH_AM29F040_LEN;
			_te3x_flash_sectors	= _FLASH_AM29F040_SECTORS;
			_te3x_flash_sector_len  = _FLASH_AM29F040_LEN/_FLASH_AM29F010_SECTORS;
			break;
		}
	    }

		// That's it, now set FLASH utilities OPEN flag
		// in order to exclude this procedure for all further
		// API function calls
        _te3x_flash_open_flag==TE3X_ON;
	r=TE3X_OK;	
      exit:
       		// execute READ/RESET command
		// in order to enable further reads
       fptr[0x5555] = 0xaa;
       fptr[0x2aaa] = 0x55;
       fptr[0x5555] = 0xf0;
       rd = *fptr;	// perform false read
       return(r);	// return error code
}


int TE3X_flash_info(unsigned *manuf_id, unsigned *dev_id,
		    unsigned *len,unsigned *sectors, unsigned *sector_len)

	// Returns FLASH chip info (manuf code, device code, etc.) via
	// suplied params pointers.
	// Returns: error code.
{ int r; 

  		// Check for FLASH API has been open already.
	  if ((r=_te3x_flash_open())!=TE3X_OK) return(_te3x_set_err(r));

	  if(manuf_id!=NULL) 	*manuf_id   =_te3x_flash_manuf_id;
	  if(dev_id!=NULL) 	*dev_id     =_te3x_flash_dev_id;
	  if(len!=NULL) 	*len	    =_te3x_flash_len;
	  if(sectors!=NULL)	*sectors    =_te3x_flash_sectors;
	  if(sector_len!=NULL)	*sector_len =_te3x_flash_sector_len;

	  return(_te3x_set_err(TE3X_OK));
}


int TE3X_flash_sector_wrprot(unsigned a)
	// Returns FLASH sector write-protection status for the sector,
	// which corresponds to address A.
	// Returns: <0 error code.
	//          0  - sector is not protected
	//          1  - sector is protected
{ int r;
  unsigned *fptr,rd,rd1;

  		// Check for FLASH API has been open already.
	if ((r=_te3x_flash_open())!=TE3X_OK) return(_te3x_set_err(r));

		// validate params
	if (a>=_te3x_flash_len) return(_te3x_set_err(TE3X_PARAM_ERR));

        	// set FLASH pointer and sector#
	fptr = (unsigned *)TE3X_FLASH_SADDR;

		// execute AUTO-SELECT COMMAND for sector protection status
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
        fptr[0x5555]=0x90;

		// read & verify sector wr-protection status (==0 or ==1)
	r=TE3X_FLASH_INV_RD_ERR;
	rd= fptr[((a/_te3x_flash_sector_len)*_te3x_flash_sector_len) +2]	& TE3X_FLASH_DATA_BITMASK;
        if(rd!=0 && rd!=1) goto exit;

	r=TE3X_OK;	// exit OK
      exit:
       		// execute READ/RESET command
		// in order to enable further reads
       fptr[0x5555] = 0xaa;
       fptr[0x2aaa] = 0x55;
       fptr[0x5555] = 0xf0;
       rd1 = *fptr;	// perform false read
       return(_te3x_set_err(r<0? r:rd));
}



int TE3X_flash_erase_sector(unsigned a,unsigned vf)
	// Erase sector, which corresponds to FLASH address A (0...
	// Verify erased sector data if VF==1.
	// returns:  error code
{ unsigned *fptr, sa, i,rd;
  int r;

  		// Check for FLASH API has been open already.
	if ((r=_te3x_flash_open())!=TE3X_OK) return(_te3x_set_err(r));

		// validate params
	if (   a>=_te3x_flash_len
	    || vf>1
	   ) return(_te3x_set_err(TE3X_PARAM_ERR));

        	// set FLASH pointer and sector#
	fptr = (unsigned *)TE3X_FLASH_SADDR;
	sa = (a/_te3x_flash_sector_len);

		// execute SECTOR ERASE command
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
        fptr[0x5555]=0x80;
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
	fptr[a]     =0x30;

        while(1)
	{ rd = fptr[a] & TE3X_FLASH_DATA_BITMASK;

		// verify read data
	  if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = fptr[a] & TE3X_FLASH_DATA_BITMASK;
                if(rd==0xff) break;	// exit OK
		return(_te3x_set_err(TE3X_FLASH_ERASE_ERR));	// error and exit
	    }	  	
	}

		// perform erased sector data verification if requested
        if(vf!=0)
	 {
	   for(i=0, fptr = (unsigned *)(TE3X_FLASH_SADDR + (sa*_te3x_flash_sector_len));
               i<_te3x_flash_sector_len;
               i++,fptr++)
		{ if((*fptr &TE3X_FLASH_DATA_BITMASK)!=0xff)
		      return(_te3x_set_err(TE3X_FLASH_ERASE_ERR));
		}
	 }
        return(_te3x_set_err(TE3X_OK));	// exit OK
}



int TE3X_flash_erase_chip(unsigned vf)
	// Erase all sectors of FLASH. Verify erased data if VF==1.
	// returns:  error code
{ unsigned *fptr,i,rd;
  int r;

  		// Check for FLASH API has been open already.
	if ((r=_te3x_flash_open())!=TE3X_OK) return(_te3x_set_err(r));

		// validate params
	if (vf>1) return(_te3x_set_err(TE3X_PARAM_ERR));

        	// set FLASH pointer
	fptr = (unsigned *)TE3X_FLASH_SADDR;

		// execute ERASE CHIP command
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
        fptr[0x5555]=0x80;
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
	fptr[0x5555]=0x10;

        while(1)
	{ rd = *fptr & TE3X_FLASH_DATA_BITMASK;

		// verify read data
	  if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = *fptr & TE3X_FLASH_DATA_BITMASK;
                if(rd==0xff) break;	// exit OK
		return(_te3x_set_err(TE3X_FLASH_ERASE_ERR));	// error and exit
	    }	  	
	}
		// perform erased chip data verification if requested
        if(vf!=0)
	 {
	   for(i=0L;i<_te3x_flash_sector_len;i++,fptr++)
		{ if((*fptr &TE3X_FLASH_DATA_BITMASK)!=0xff)
		      return(_te3x_set_err(TE3X_FLASH_ERASE_ERR));
		}
	 }
        return(_te3x_set_err(TE3X_OK));	// exit OK
}


int TE3X_flash_write(unsigned a, unsigned d)
	// writes D (bits 0..7 are valid) to FLASH addr #A
	// returns:  error code
{ unsigned *fptr,rd;
  int r;

  		// Check for FLASH API has been open already.
	if ((r=_te3x_flash_open())!=TE3X_OK) return(_te3x_set_err(r));

		// validate params
	if (   a>=_te3x_flash_len
	    || d>TE3X_FLASH_DATA_BITMASK
	   ) return(_te3x_set_err(TE3X_PARAM_ERR));

        	// set FLASH pointer
	fptr = (unsigned *)TE3X_FLASH_SADDR;

		// verify that FLASH contents at address A
		// is OK to program data D, i.e. '1' bits in D
		// shall match '1' bits in FLASH contents
	if ((fptr[a] & d)!=d)
	    return(_te3x_set_err(TE3X_FLASH_CANNOT_WR_ERR));

		// execute PROGRAM BYTE command
        fptr[0x5555]=0xaa;
        fptr[0x2aaa]=0x55;
        fptr[0x5555]=0xa0;
        fptr[a]=d ;

		// verify for program ends and errors
        while(1)
	{ rd = (fptr[a]) & TE3X_FLASH_DATA_BITMASK;

		// verify read data
	  if(rd==d) return(_te3x_set_err(TE3X_OK));		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = fptr[a] & TE3X_FLASH_DATA_BITMASK;
                if(rd==d) return(_te3x_set_err(TE3X_OK));	// exit OK
		return(_te3x_set_err(TE3X_FLASH_WR_ERR));	// error and exit
	    }	  	
	}
}


int TE3X_flash_read(unsigned a)
	// reads/returns data from FLASH addr #A
	// returns: <0 - error occured
	//	    >=0 - FLASH read data
	//
	// Note: TE3X_FLASH_READ function does not generally require
	//       FLASH WRITE enabled via jumper J2 in order to
	//       have FLASH utilities running perfectly.
	//       In case FLASH utilities are open, then A address is
	//       verified againg particular FLASH length, otherwise
	//       maximum FLASH/EPROM length is used.

{ unsigned *fptr;
  int r;

		// execute FLASH OPEN procedure in order to activate
		// FLASH utilities, however ignore returned error code
	r=_te3x_flash_open();	

		// validate function params
	if (    (_te3x_flash_open_flag==TE3X_ON  && a>=_te3x_flash_len)
	      ||(_te3x_flash_open_flag==TE3X_OFF && a>=TE3X_FLASH_LEN)
	   ) return(_te3x_set_err(TE3X_PARAM_ERR));

	if (_te3x_flash_open_flag==TE3X_ON)
		_te3x_last_err=TE3X_OK;	// no errors if FLASH OPEN is OK

        	// set FLASH pointer and read data
	fptr = (unsigned *)TE3X_FLASH_SADDR;
	return(fptr[a] & TE3X_FLASH_DATA_BITMASK);
}

//********************************************************************

